#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <EEPROM.h>
#include <math.h>

// defines
#define DAC 0x3C            // Expander D/A Converter (0x38,0x39,0x3C,0x3D)
#define LCDEX 0x3A          // Expander LCD
#define SensorAdresse 0x48  // Baseadress first LM75
#define SREFLOW 0           // LM75 Reflow-oven
#define SCOOLER 5           // LM75 Cooling chamber Ab V2 (1 0x49,3 0x4B,5 0x4D,7 0x4F)
#define TEMP 0              // Temp register
#define UNTEN 2             // Lower threshold
#define OBEN 3              // Upper threshold
#define MAXTEMP 40.00       // Maximum value for the OS-output
#define STATUS_GN 0         // Status LED green
#define STATUS_YE 1         // Status LED yellow / red
#define STATUS_RD 2         // Status LED red
#define STATUS_OFF 3        // Status LED off

// define Time for LM75
/* !!!! Under certain circumstances, the temperature may be read out too quickly. This is due to the LM75 used, type B is faster than type A. !!! */
#define LM75_time 500  // Time in ms between re-reading for K8 and K9
#define LM75_show 0    // Show temperature during commissioning
#define S_RESET_SKIP 0

// define INPUTs & OUTPUTs
#define S1 2
#define S2 3
#define S3 4
#define CONVS 7   // Conveyor belt position
#define SMODE 11  // Status mode B2 ? P

#define SLED1 5      // Status LED
#define SLED2 6      // Status LED
#define CONVB 8      // Conveyor belt enable
#define FAN 9        // fan enable
#define SIMH 10      // simulate heater
#define ENABLE_H 12  // enable heater

LiquidCrystal_I2C lcd(LCDEX, 16, 4);
#define LCDOFFSET -4  // -4 by 16 Char at line, 0 by 20 Char at line

// Functions
// Output CheckCodes and ErrorCodes
// Meaning of the codes can be taken from the table.
void statusCode(const String &);
void checkCode(const String &);
void statusLED(int);
boolean readSMode(void);
void checkI2CBus(int);
void shutdown(int);
void goToMenu(void);

// LCD Output and waiting of S2
void outputLCD(const String &, const String &);
void readSwitch(int);

// Output expander (Input of D/A Converter)
void sendDigitalValue(byte ADDR, byte value);

// LM75
double get_LM75_temperature(int device, int regx);
void set_LM75_config(int device, byte value);
byte get_LM75_config(int device);
void set_LM75_threshold(int device, byte regx, double grad);

// Measurements ...
void goToDA(void);
void goToI2C(void);
void goToInit(void);
void goToSym(void);

// global variables
char dataString[7];  // read temperature processed as a string: (-xx)x.x
double tmpRoom1, tmpRoom2;

void (*resetFunc)(void) = 0;  //declare reset function at address 0

void setup() {
  // put your setup code here, to run once:
  int sMode = 0;
  unsigned long previousMillis = 0;
  unsigned long currentMillis = 0;
  boolean pinIn[11] = { true, true, true, false, false, true, false, false, false, true, false };
  int i2c_address[4] = { DAC, LCDEX, (SensorAdresse + SREFLOW), (SensorAdresse + SCOOLER) };
  Wire.begin();
  Serial.begin(9600);
  while (!Serial)
    ;
  Serial.println("\nProgram transfer complete! V3");
  Serial.println("You can unplug the device!");
  Serial.end();
  analogReference(EXTERNAL);
  for (int i = 2; i < 13; i++) {
    if (pinIn[(i - 2)]) {
      pinMode(i, INPUT);
    } else {
      pinMode(i, OUTPUT);
      digitalWrite(i, LOW);
    }
  }
  sendDigitalValue(DAC, 0x00);   // OPV set output voltage to min
  digitalWrite(CONVB, HIGH);     // disable conveyor belt
  digitalWrite(ENABLE_H, HIGH);  // disable heater
  statusLED(STATUS_OFF);
  lcd.init();  // initialize the lcd
  lcd.init();
  lcd.setCursor(0, 0);
  lcd.print("V3");
  outputLCD(" Willkommen zur  ", " Winter Pr\365fung ");
  delay(1200);
  outputLCD("  Booting  0 %   ", "Init PORTs, I2C ");
  for (int i = 1; i < 4; i++, delay(500))  //Delay to temper the sensors
  {
    lcd.setCursor(11, 1);
    lcd.print(i);
  }
  outputLCD("  Booting 5 %   ", "                ");
  for (int i = 10, e = 0; i < 81; i++, delay(70))  //Delay to temper the sensors
  {
    lcd.setCursor(10, 1);
    lcd.print(int(i * 1.25));
    lcd.print(" %  ");
    if (!(i % 10) && e < 4) {
      lcd.setCursor(LCDOFFSET, 2);
      lcd.print("Check I2C: 0x");
      lcd.print(i2c_address[e], HEX);
      checkI2CBus(i2c_address[e]);
      e++;
    } else if (!(i % 10) && e == 4) {
      // Set thresholds
      lcd.setCursor(LCDOFFSET, 2);
      lcd.print("Config LM75: 0  ");
      // Heating room
      tmpRoom1 = get_LM75_temperature(SREFLOW, TEMP);
      //#Serial.print("\nConfig LM75 1: ");
      set_LM75_threshold(SREFLOW, UNTEN, MAXTEMP - 3.00);
      set_LM75_threshold(SREFLOW, OBEN, MAXTEMP);
      //#Serial.println("OK");
      delay(400);
      lcd.setCursor(LCDOFFSET + 13, 2);
      lcd.print(SCOOLER);
      // Cooling room
      //#Serial.print("Config LM75 2: ");
      tmpRoom2 = get_LM75_temperature(SCOOLER, TEMP);
      set_LM75_threshold(SCOOLER, UNTEN, MAXTEMP - 3.00);
      set_LM75_threshold(SCOOLER, OBEN, MAXTEMP);
      //#Serial.println("OK");
      e++;
    } else if (i == 65 && e == 5) {
      lcd.setCursor(LCDOFFSET, 2);
      lcd.print("Checking Belt:");
      //#Serial.print("Checking conveyor belt: ");
      digitalWrite(CONVB, LOW);  // enable conveyor belt
      currentMillis = millis();
      previousMillis = currentMillis;
      while (!digitalRead(CONVS) && ((currentMillis - previousMillis) <= 10000)) {
        currentMillis = millis();
      }
      if ((currentMillis - previousMillis) >= 10000) {
        //#Serial.println("ERROR");
        lcd.print("NO");
        statusCode("ECON");
        shutdown(1);
      } else {
        //#Serial.println("OK");
        lcd.print("OK");
        i = 75;
        digitalWrite(CONVB, HIGH);  // disable conveyor belt
        for (int a = 0; a < 10; a++, digitalWrite(CONVB, HIGH)) {
          delay(20);
          digitalWrite(CONVB, LOW);
          delay(20);
        }
        e++;
      }
    }
  }
  digitalWrite(ENABLE_H, LOW);  // enable heater
  if (!readSMode()) statusLED(STATUS_GN);
  else statusLED(STATUS_OFF);
  outputLCD("                ", "                ");
}

void loop() {
  if (!readSMode()) {
    lcd.setCursor(LCDOFFSET, 3);
    lcd.print("    | MENU |    ");
    delay(100);
    if (!digitalRead(S2)) goToMenu();
  } else {
    goToMenu();
  }
}

void checkCode(const String &n1) {
  lcd.setCursor(0, 0);
  lcd.print(n1);
  return;
}

void checkI2CBus(int I2C_address) {
  Wire.beginTransmission(I2C_address);
  int answer = Wire.endTransmission();
  if (answer == 2) {
    //#Serial.print("\nNo Device found at address: 0x");
    //#Serial.print(I2C_address, HEX);
    statusCode("EI2C");
    shutdown(1);
  } else if (!answer) {
    //#Serial.print("\nDevice found at address: 0x");
    //#Serial.print(I2C_address, HEX);
  }
  return;
}

void goToDA(void) {
  int menu = 0;
  int program[6][5] = { { 0, 10, 50, 60, 40 }, { 10, 35, 100, 60, 20 }, { 100, 60, 10, 10, 35 }, { 25, 15, 100, 100, 10 }, { 0, 0, 0, 0, 0 }, { 100, 100, 100, 100, 100 } };
  digitalWrite(ENABLE_H, HIGH);  // disable heater
  statusLED(STATUS_YE);
  checkCode("MESS");
  outputLCD("D/A-Messung ist ", "aktiv!   PROG: 1");
  lcd.setCursor(LCDOFFSET, 3);
  lcd.print(" <  |  END |  > ");
  do {
    lcd.setCursor(LCDOFFSET + 15, 2);
    if (!digitalRead(S3)) {
      readSwitch(S3);
      menu++;
    } else if (!digitalRead(S1)) {
      readSwitch(S1);
      menu--;
    }
    if (menu < 0) menu = 0;
    if (menu > 5) menu = 5;
    lcd.print((menu + 1));
    for (int i = 0; i < 5; i++, delay(10)) {
      sendDigitalValue(DAC, (int)map(program[menu][i], 0, 100, 0, 255));
    } /**/
    delay(10);
  } while (digitalRead(S2));
  lcd.setCursor(LCDOFFSET, 2);
  lcd.print("abgeschlossen!  ");
  readSwitch(S2);
  sendDigitalValue(DAC, 0x00);
  digitalWrite(ENABLE_H, LOW);  // enable heater
  statusLED(STATUS_GN);
  return;
}

void goToI2C(void) {
  digitalWrite(ENABLE_H, HIGH);  // disable heater
  statusLED(STATUS_YE);
  checkCode("MESS");
  lcd.setCursor(LCDOFFSET, 2);
  lcd.print("aktiv!          ");
  lcd.setCursor(LCDOFFSET, 3);
  lcd.print("             END");
  do {
    sendDigitalValue(DAC, 0x90);
    delay(5);
  } while (digitalRead(S3));
  sendDigitalValue(DAC, 0x00);
  digitalWrite(ENABLE_H, LOW);  // enable heater
  lcd.setCursor(LCDOFFSET, 2);
  lcd.print("abgeschlossen!  ");
  readSwitch(S3);
  statusLED(STATUS_GN);
  return;
}

void goToInit(void) {
  unsigned long previousMillis = 0;
  unsigned long currentMillis = 0;
  digitalWrite(ENABLE_H, HIGH);  // disable heater
  statusLED(STATUS_OFF);
  checkCode("INIT");
  lcd.setCursor(LCDOFFSET, 2);
  lcd.print("aktiv!          ");
  lcd.setCursor(LCDOFFSET, 3);
  lcd.print("             > ");
  readSwitch(S3);
  statusLED(STATUS_GN);
  outputLCD("Die Leuchte -P3", "leuchtet.");
  readSwitch(S3);
  statusLED(STATUS_RD);
  outputLCD("Die Leuchte -P2", "");
  readSwitch(S3);
  statusLED(STATUS_OFF);
  digitalWrite(CONVB, LOW);
  outputLCD("Abgl. an -MP1   ", "         ");
  readSwitch(S3);
  digitalWrite(CONVB, HIGH);
  sendDigitalValue(DAC, 0xFF);
  outputLCD("Abgl. an -MP3   ", "");
  readSwitch(S3);
  sendDigitalValue(DAC, 0x00);
  outputLCD("Pr\365fung des -K8 ", "                ");
  lcd.setCursor(LCDOFFSET, 3);
  lcd.print("TEST            ");
  tmpRoom1 = get_LM75_temperature(SREFLOW, TEMP);  // get current temperature
  readSwitch(S1);
  lcd.setCursor(LCDOFFSET, 3);
  lcd.print("    ");
  if (LM75_show) {
    lcd.setCursor(11, 0);
    lcd.print(tmpRoom1);
  }
  digitalWrite(ENABLE_H, LOW);     // enable heater
  sendDigitalValue(DAC, 0xFF);     // set 100% PWM
  currentMillis = millis();        // get current millis
  previousMillis = currentMillis;  // save to prevoius millis
  int sec = 9, wert = 1;
  lcd.setCursor(LCDOFFSET, 2);
  lcd.print("10  ");
  while (1.50 >= (get_LM75_temperature(SREFLOW, TEMP) - tmpRoom1) && ((currentMillis - previousMillis) <= 10000))  // check to get higher temperature in 10 sec.
  {
    if ((currentMillis - previousMillis) >= (wert * 1000)) {
      wert++;
      lcd.setCursor(LCDOFFSET, 2);
      lcd.print(sec);
      lcd.print("  ");
      if (LM75_show) {
        lcd.setCursor(11, 0);
        lcd.print(get_LM75_temperature(SREFLOW, TEMP));
      }
      sec--;
    }
    currentMillis = millis();
    delay(LM75_time);
  }
  sendDigitalValue(DAC, 0x00);
  if ((currentMillis - previousMillis) >= 10000) {
    lcd.setCursor(11, 0);
    lcd.print(" ETH1");
    //lcd.setCursor(LCDOFFSET, 3);
    //lcd.print("                ");
    shutdown(2);
  }
  lcd.setCursor(LCDOFFSET, 2);
  lcd.print("beendet!       ");
  lcd.setCursor(LCDOFFSET, 3);
  lcd.print("              > ");
  digitalWrite(ENABLE_H, HIGH);  // disable heater
  readSwitch(S3);
  outputLCD("Pr\365fung des -K9 ", "                ");
  lcd.setCursor(LCDOFFSET, 3);
  lcd.print("TEST            ");
  tmpRoom2 = get_LM75_temperature(SCOOLER, TEMP);  // get current temperature
  readSwitch(S1);
  lcd.setCursor(LCDOFFSET, 3);
  lcd.print("    ");
  if (LM75_show) {
    lcd.setCursor(11, 0);
    lcd.print(tmpRoom2);
  }
  digitalWrite(SIMH, HIGH);
  currentMillis = millis();        // get current millis
  previousMillis = currentMillis;  // save to prevoius millis
  sec = 10;
  wert = 1;
  while (1.50 >= (get_LM75_temperature(SCOOLER, TEMP) - tmpRoom2) && ((currentMillis - previousMillis) <= 10000))  // check to get higher temperature in 10 sec.
  {
    if ((currentMillis - previousMillis) >= (wert * 1000)) {
      wert++;
      lcd.setCursor(LCDOFFSET, 2);
      lcd.print(sec);
      lcd.print("  ");
      if (LM75_show) {
        lcd.setCursor(11, 0);
        lcd.print(get_LM75_temperature(SCOOLER, TEMP));
      }
      sec--;
    }
    currentMillis = millis();
    delay(LM75_time);
  }
  if ((currentMillis - previousMillis) >= 10000) {
    lcd.setCursor(11, 0);
    lcd.print(" ETH2");
    //lcd.setCursor(LCDOFFSET, 3);
    //lcd.print("                ");
    shutdown(2);
  }
  digitalWrite(SIMH, LOW);
  lcd.setCursor(LCDOFFSET, 2);
  lcd.print("beendet!       ");
  lcd.setCursor(11, 0);
  lcd.print("     ");
  lcd.setCursor(LCDOFFSET, 3);
  lcd.print("              > ");
  readSwitch(S3);
  outputLCD("L\365fter an -X10 ", "wird angesteuert!");
  digitalWrite(FAN, HIGH);
  readSwitch(S3);
  digitalWrite(FAN, LOW);
  digitalWrite(ENABLE_H, LOW);  // enable heater
  outputLCD("Inbetriebnahme ", "abgeschlossen!  ");
  lcd.setCursor(LCDOFFSET + 14, 3);
  lcd.print("  ");
  while (1)
    ;
  return;
}

void goToMenu(void) {
  if (!readSMode()) readSwitch(S2);
  int menu = 0;
  boolean sMode = readSMode();
  menu = (!sMode ? 1 : 0);
  while (1) {
    lcd.setCursor(LCDOFFSET, 3);
    if (!sMode)
      lcd.print("BACK");
    else
      lcd.print("    ");
    lcd.print(" |START| ");
    lcd.print((!sMode ? "-> " : "   "));
    do {
      if (!digitalRead(S3) && !sMode) {
        readSwitch(S3);
        menu++;
        menu = (menu > 4 ? 1 : menu);
      } else if (!digitalRead(S1) && !sMode) {
        readSwitch(S1);
        outputLCD("                ", "                ");
        return;
      }
      lcd.setCursor(0, 1);
      switch (menu) {
        case 0: lcd.print("Inbetriebnahme "); break;
        case 1: lcd.print("Heizkurve      "); break;
        case 2: lcd.print("I2C-Messung    "); break;
        case 3: lcd.print("Status-LEDs    "); break;
        case 4: lcd.print("Simulation     "); break;
      }

    } while (digitalRead(S2));
    readSwitch(S2);

    switch (menu) {
      case 0: goToInit(); break;
      case 1: goToDA(); break;
      case 2: goToI2C(); break;
      case 3: goToStatusLED(); break;
      case 4: goToSym(); break;
    }
    checkCode("    ");
    outputLCD("                ", "                ");
  }
  return;
}

void goToStatusLED(void) {
  digitalWrite(ENABLE_H, HIGH);  // disable heater
  statusLED(STATUS_OFF);
  checkCode("ILED");
  lcd.setCursor(LCDOFFSET, 3);
  lcd.print("             > ");
  readSwitch(S3);
  statusLED(STATUS_GN);
  outputLCD("Leuchtet -P3? ", "          ");
  lcd.setCursor(LCDOFFSET, 3);
  lcd.print("     NEIN | JA ");
  while (digitalRead(S2) && digitalRead(S3))
    ;
  if (!digitalRead(S2)) {
    statusCode("ELED");
    lcd.setCursor(LCDOFFSET, 3);
    lcd.print("                ");
    shutdown(0);
  }
  readSwitch(S3);
  statusLED(STATUS_YE);
  outputLCD("Leuchtet -P1?", "");
  while (digitalRead(S2) && digitalRead(S3))
    ;
  if (!digitalRead(S2)) {
    statusCode("ELED");
    lcd.setCursor(LCDOFFSET, 3);
    lcd.print("                ");
    shutdown(0);
  }
  readSwitch(S3);
  statusLED(STATUS_RD);
  outputLCD("Leuchtet -P2?", "");
  while (digitalRead(S2) && digitalRead(S3))
    ;
  if (!digitalRead(S2)) {
    statusCode("ELED");
    lcd.setCursor(LCDOFFSET, 3);
    lcd.print("                ");
    shutdown(0);
  }
  readSwitch(S3);
  statusLED(STATUS_OFF);
  outputLCD("Inbetriebnahme  ", "abgeschlossen!  ");
  lcd.setCursor(LCDOFFSET, 3);
  lcd.print("            END");
  readSwitch(S3);
  statusLED(STATUS_GN);
  digitalWrite(ENABLE_H, LOW);  // enable heater
  return;
}

void goToSym(void) {
  int i = 0;
  statusLED(STATUS_YE);
  checkCode("MESS");
  lcd.setCursor(LCDOFFSET, 2);
  lcd.print("gestartet!      ");
  lcd.setCursor(LCDOFFSET, 3);
  lcd.print("            END");
  tmpRoom1 = get_LM75_temperature(SREFLOW, TEMP);  // get current temperature
  sendDigitalValue(DAC, 0xB2);
  while (get_LM75_temperature(SREFLOW, TEMP) < MAXTEMP && digitalRead(S3)) delay(20);
  sendDigitalValue(DAC, 0x00);
  lcd.setCursor(LCDOFFSET, 2);
  lcd.print("gestoppt! ");
  if (get_LM75_temperature(SREFLOW, TEMP) > MAXTEMP) {
    statusLED(STATUS_RD);
    statusCode("ESIM");
    while (get_LM75_temperature(SREFLOW, TEMP) - 3.00 > tmpRoom1) delay(200);
  }
  /*
  tmpRoom2 = get_LM75_temperature(SCOOLER, TEMP); // get current temperature
  digitalWrite(SIMH, HIGH);
  digitalWrite(CONV, LOW);
  while(i < 3)
  {
    delay(50);
    if(get_LM75_temperature(SRELOW, TEMP) > MAXTEMP)
    {
      digitalWrite(ENABLE_H, HIGH);
      i = 5;
    }
    if(digitalRead(CONS))
    {
        i++;
    }
    if(i == 3)
      sendDigitalValue(DAC, 0x00); 
  }
  digitalWrite(SIMH, LOW);
  while(!digitalRead(CONS));
  digitalWrite(CONVB, HIGH);    // disable conveyor belt
  for(int a = 0;a < 10;a++, digitalWrite(CONVB, HIGH)) 
  {
    delay(5);
    digitalWrite(CONVB, LOW);
    delay(10); 
  }
  /**/
  readSwitch(S3);
  statusLED(STATUS_GN);
  return;
}

void outputLCD(const String &n1, const String &n2) {
  lcd.setCursor(0, 1);
  lcd.print(n1);
  lcd.setCursor(LCDOFFSET, 2);
  lcd.print(n2);
  return;
}

boolean readSMode() {
  return digitalRead(SMODE);
}

void readSwitch(int swID) {
  while (digitalRead(swID))
    ;
  while (!digitalRead(swID)) delay(100);
  return;
}

void shutdown(int ausgabe) {
  int err = 0;
  checkCode("    ");
  switch (ausgabe) {
    case 0: outputLCD("System shutdown ", "                "); break;
    case 1:
      lcd.setCursor(0, 1);
      lcd.print("- Stop Booting -");
      break;
    case 2:
      err = 1;
      outputLCD("System shutdown ", "                ");
      break;
  }
  sendDigitalValue(DAC, 0x00);   // OPV set output voltage to min
  digitalWrite(CONVB, HIGH);     // disable conveyerbelt
  digitalWrite(FAN, HIGH);       // enable fan
  digitalWrite(SIMH, LOW);       // disable heating simulation
  digitalWrite(ENABLE_H, HIGH);  // disable heater
  statusLED(STATUS_RD);
  if (S_RESET_SKIP) {
    lcd.setCursor(LCDOFFSET, 3);
    lcd.print("    | RESET |   ");
    if (err) {
      lcd.setCursor(LCDOFFSET + 14, 3);
      lcd.print(">>");
    }
    while (digitalRead(S2) && digitalRead(S3)) delay(10);
    if (!digitalRead(S2)) err = 0;
    if (!err)
      resetFunc();  //call reset
    else {
      while (!digitalRead(S3)) delay(100);
      digitalWrite(FAN, LOW);  // enable fan
      statusLED(STATUS_OFF);
      statusCode("    ");
    }
  } else
    while (1)
      ;
  return;
}

void statusCode(const String &n1) {
  lcd.setCursor(12, 0);
  lcd.print(n1);
  return;
}

void statusLED(int sled) {
  if (readSMode()) {
    switch (sled) {
      case 0:
        digitalWrite(SLED1, LOW);
        digitalWrite(SLED2, HIGH);
        break;
      case 1:
        digitalWrite(SLED1, HIGH);
        digitalWrite(SLED2, LOW);
        break;
      case 2:
        digitalWrite(SLED1, HIGH);
        digitalWrite(SLED2, LOW);
        break;
      case 3:
        digitalWrite(SLED1, HIGH);
        digitalWrite(SLED2, HIGH);
        break;
    }
  } else {
    switch (sled) {
      case 0:
        digitalWrite(SLED1, HIGH);
        digitalWrite(SLED2, HIGH);
        break;
      case 1:
        digitalWrite(SLED1, HIGH);
        digitalWrite(SLED2, LOW);
        break;
      case 2:
        digitalWrite(SLED1, LOW);
        digitalWrite(SLED2, HIGH);
        break;
      case 3:
        digitalWrite(SLED1, LOW);
        digitalWrite(SLED2, LOW);
        break;
    }
  }
  return;
}

void sendDigitalValue(byte ADDR, byte value) {
  Wire.beginTransmission(ADDR);
  Wire.write(value);
  Wire.endTransmission();
}

// LM75 Temperatur auslesen. Device = 0-7, regx = TEMP, OBEN, UNTEN (Registerauswahl)
double get_LM75_temperature(int device, int regx) {
  int8_t msb;
  int8_t lsb;
  int8_t msb1;
  Wire.beginTransmission(SensorAdresse + device);
  Wire.write(regx);
  Wire.endTransmission();
  Wire.beginTransmission(SensorAdresse + device);
  Wire.requestFrom(SensorAdresse + device, 2);
  if (Wire.available()) {
    msb1 = Wire.read();
    msb = msb1 << 1;  // Remove sign bit, remaining value is now twice as large
    lsb = Wire.read();
  }
  // The highest bit indicates whether 0.5 degrees are added
  lsb = (lsb & 0x80) >> 7;
  Wire.endTransmission();
  if (msb1 < 0x80) {               // Is the value positive?
    return double(msb + lsb) / 2;  // positive value
  } else {
    return double(msb + lsb) / 2 - 128;  // negative value
  }
}

// LM75 set configuration
void set_LM75_config(int device, byte value) {
  Wire.beginTransmission(SensorAdresse + device);
  Wire.write(1);  // Select register config
  Wire.write(value);
  Wire.endTransmission();
}

// LM75 read configuration, device = 0-7
byte get_LM75_config(int device) {
  byte reg;
  Wire.beginTransmission(SensorAdresse + device);
  Wire.write(1);  // Select register config
  Wire.endTransmission();
  Wire.requestFrom(SensorAdresse + device, 1);
  if (Wire.available()) {
    reg = Wire.read();
  }
  Wire.endTransmission();
  return reg;
}

// LM75 set threshold, device = 0-7, regx = Value, Degree as double
void set_LM75_threshold(int device, byte regx, double grad) {
  int answer = 0;
  int8_t msb;
  int8_t lsb = 0;
  uint8_t y = 0;
  boolean neg = false;
  if (grad < 0) {
    msb = abs(int(grad)) + 128;
  } else {
    msb = abs(int(grad));
  }
  if (grad - abs(int(grad)) > 0) {
    lsb = 0x80;
  }
  Wire.beginTransmission(SensorAdresse + device);
  Wire.write(regx);  // select upper or lower register
  Wire.write(msb);
  Wire.write(lsb);
  answer = Wire.endTransmission();
  if (answer) {
    statusCode("ETC");
    lcd.print(device);
    shutdown(1);
  }
}
